#!/usr/bin/env python3

import sys
from collections import deque

def wrong(msg):
    print(msg)
    sys.exit(1)

try:
    with open(sys.argv[2], 'rb') as f: their_out = f.read().split()
    their_out = [line.decode('unicode_escape') for line in their_out]
except Exception:
    wrong('Wrong answer: failed to parse output, use 7-bit ASCII only')

hard = int(sys.argv[1])

# setting up constants
if hard:
    rows, columns = 7, 7
else:
    rows, columns = 3, 3

if hard:
    directions = [(-1, -1), (-1, 0), (-1, 1),
                  (0, -1),           (0, 1),
                  (1, -1),  (1, 0),  (1, 1)]
else:
    directions = [          (-1, 0),
                  (0, -1),           (0, 1),
                            (1, 0)]

their_rows = 0
their_columns = 0

# checking formatting
try:
    their_rows, their_columns = int(their_out[0]), int(their_out[1])
except Exception:
    wrong('Wrong answer: Input must start with two integers')

their_out = their_out[2:]

if len(their_out) != their_rows:
    wrong('Wrong answer')

for row in their_out:
    if len(row) != their_columns:
        wrong('Wrong answer')

# checking size
if their_rows != rows:
    wrong('Wrong answer')

if their_columns != columns:
    wrong('Wrong answer')

# find start and finish
start_pos =  (-1,-1)
finish_pos = (-1,-1)

for x in range(0, len(their_out)):
    row = their_out[x]
    for y in range(0, len(row)):
        if their_out[x][y] == 'A':
            if start_pos != (-1,-1):
                wrong('Wrong answer: incorrectly formatted grid')
            start_pos = (x, y)

        if their_out[x][y] == 'B':
            if finish_pos != (-1,-1):
                wrong('Wrong answer: incorrectly formatted grid')
            finish_pos = (x, y)

if start_pos == (-1,-1) or finish_pos == (-1,-1):
    wrong('Wrong answer: incorrectly formatted grid')

# read grid
def to_num(c):
    if c == '#':
        return 1
    if c in ['.', 'A', 'B']:
        return 0
    wrong('Wrong answer: incorrectly formatted grid')

grid = [[to_num(pos) for pos in row] for row in their_out]
# print(grid)

def add(vec1, vec2):
    return (vec1[0] + vec2[0], vec1[1] + vec2[1])

def next_pos(pos, direction, grid):
    npos = add(pos, direction)
    while True:
        if npos[0]<0 or npos[1]<0 or npos[0] >= rows or npos[1] >= columns:
            return None
        if not grid[npos[0]][npos[1]]:
            return npos
        npos = add(npos, direction)

# BFS
def bfs(grid, dirs):
    dist = [[None for pos in row] for row in grid]

    q = deque()
    q.append( (start_pos, 0) )

    while q:
        cur = q.popleft()
        if dist[cur[0][0]][cur[0][1]]:
            continue
        dist[cur[0][0]][cur[0][1]] = cur[1]
        for d in dirs:
            npos = next_pos(cur[0], d, grid)
            if npos:
                q.append( (npos, cur[1]+1) )
    return dist[finish_pos[0]][finish_pos[1]]

# check that path exists
if not bfs(grid, directions):
    wrong('Wrong answer')

# check that no path with fewer directions exists
for i in range(len(directions)):
    dirs_miss_one = directions.copy()
    del dirs_miss_one[i]
    if bfs(grid, dirs_miss_one):
        wrong('Wrong answer')

print('Huzzah!')
